function [A,B]=UFC(N,D,Q)

% UNITY FEEDBACK CONTROLLER
%
% Coelho, J.P. (2005/2006). "CONTROLO DIGITAL - Material de Apoio s Aulas". Instituto Politcnico de Bragana, 
% Escola Superior de Tecnologia e Gesto.(2005/2006)
%
% WEB: www.ipb.pt/~jpcoelho
% E-mail: jpcoelho@ipb.pt
%
% Copyright 2006
%
% Este "script" pode ser usado livremente desde que devidamente referenciado.
% Este cdigo apenas pode ser usado para fins educacionais. 
% Qualquer utilizao pblica desta rotina requer a autorizao expressa do autor 

%
% [A,B]=UFC(N,D,Q)
%
% PROJECTO DE UM CONTROLADOR RECORRENDO AO POSICIONAMENTO DE PLOS
%
% DADO UM PROCESSO COM FUNO DE TRANSFERNCIA
%        N(s)
% G(s)=---------
%        D(s)
%
% PRETENDE-SE ESTIMAR UM CONTROLADOR
%        A(s)
% K(s)=---------
%        B(s)
%
% DE MODO QUE A FUNO DE TRANSFERNCIA DE MALHA FECHADA (COM
% REALIMENTAO UNITRIA) SEJA:
%          P(s) 
% Gcl(s)=--------
%          Q(s)
%
% A FUNO DE TRANSFERNCIA DE MALHA FECHADA DA CASCATA PROCESSO+CONTROLADOR
% :
%
%    G(s)K(s)          N(s)A(s) 
%  ------------ = ------------------
%   1+G(s)K(s)     D(s)B(s)+N(s)A(s)
% 
% PRETENDE-SE QUE ESSA FUNO DE TRANSFERNCIA SEJA IGUAL A Gcl(s) LOGO, IGUALANDO
% AMBAS AS EXPRESSES VEM QUE:
%  /
%  | P(s)=N(s)A(s)
% <
%  | Q(s)=D(s)B(s)+N(s)A(s)
%  \
%
% COMO SE PODE OBSERVAR APENAS SE TEM CONTROLO NA POSIO DOS PLOS LOGO A RESPOSTA TRANSITORIA
% PODE NO SER EXACTAMENTE AQUELA QUE DESEJAMOS.
%
% COMO O PROCESSO  CONSIDERADO FIXO A EQUAO A RESOLVER  DO TIPO:
% 
% Ax+By=C designada normalmente por equao de Diophantine
%
% SE O SISTEMA  DE ORDEM n ENTO O CONTROLADOR SER DE ORDEM n-1 E A ORDEM DO POLINMIO
% Q(s) SER DE ORDEM 2n-1


if nargin~=3 | ~isnumeric(N) | ~isnumeric(D)| ~isnumeric(Q) | min(size(N))>1| min(size(D))>1| min(size(Q))>1
   disp('Erro: Parmetros de Entrada Invlidos');
   return
end
% Verificar se o sistema  prprio
np=length(roots(D));
nz=length(roots(N));
if np<nz,
   disp('Erro: O Sistema  Imprprio');
   return
elseif np>nz
   N=[zeros(1,np-nz) N];
end
% Plos desejados...
ncl=length(Q)
if ncl<2*np-1,
   disp('Expandindo os Modos do Sistema...');
   np=2*np-1-ncl;
   mx=max(abs(real(Q)));
   Q=[Q -mx*10.^(1:np)]; %coloco polos redundantes dcadas  frente...
end
Q
pQ=poly(Q)
%SOLUO DA EQUAO S(N,D)X=F
F=fliplr(pQ)';
S=sylvester(N,D);




X=S\F;
X=reshape(X,2,length(X)/2);
A=fliplr(X(1,:));
B=fliplr(X(2,:));


function S=sylvester(N,D)

% Constroi a matriz de Sylvester...
%
% Atenda  forma como o produto de Kronecker  usado 
% para evitar ciclos...
%                        ;)
%
s1=[fliplr(D)' fliplr(N)'];
n=length(roots(D));
i=ones(2*n,1)*(1:n+1);
i=i';i=i(:);
ii=ones(2*(n+1),1)*(0:n-1);
ii=ii(:);i=i+ii;
j=ones(n+1,1)*(1:2*n);j=j(:);
S=full(sparse(i,j,kron(ones(n,1),s1(:))));






