% INFLUNCIA DO FILTRO ANTI-ALIASING NAS PERTURBAE FORA DE BANDA
%
% Coelho, J.P. (2005/2006). "CONTROLO DIGITAL - Material de Apoio s Aulas". Instituto Politcnico de Bragana, 
% Escola Superior de Tecnologia e Gesto.(2005/2006)
%
% WEB: www.ipb.pt/~jpcoelho
% E-mail: jpcoelho@ipb.pt
%
% Copyright 2006
%
% Este "script" pode ser usado livremente desde que devidamente referenciado.
% Este cdigo apenas pode ser usado para fins educacionais. 
% Qualquer utilizao pblica desta rotina requer a autorizao expressa do autor 


clear all;close all;
s=tf('s');

% Sistema Analgico
%            76
% G(s)=--------------
%       (s+1)(s+3)^2
%
% Deve verifica, em malha fechada, os seguintes critros de desempenho:
% 	- ess <10%
%	- Pm ~ 45
%	- BW<2 rad/s

% Funo de Transferencia
G_s=76/((s+1)*(s+3)^2);
% Controlador Analgico (Atraso de Fase)
K_s=1.851*(0.494*s+1)/(14.49*s+1);
% Funo de Transferncia de Malha Fechada
Gcl_s=feedback(G_s*K_s,1);
% Largura de Banda em Malha Fechada
BW=1.4;    % rad/s
% Frequencia de cruzamento de ganho
wgc=0.83;  %  rad/s
% Frequncia e Perodo de Amostragem (Valor Tipico!)
wo=30*BW;  % rad/s
T=2*pi/wo; % s
z=tf('z',T); % Objecto tipo TF/TF object 
% Funo de Transferncia do Filtro Anti-Aliasing
Gf_s=(wo/2)/(s+(wo/2));

%             ______     _________     ______
% R(z)-O---->| K(z) |-->| Gzoh(s) |-->| G(s) |-->O-->Y(s)
%      |      ------     ---------     ------    |
%      |                     _______     +       |
%       -------------- \----| Gf(s) |<--O--------
%                            -------    |+
%                                       N(s)

% N(s) - sinal sinusoidal com frequncia 61 rad/s e amplitude 0.3
%
%         18.3        alfa
% N(s)=----------=-------------
%       s^2+86^2   s^2+delta^2
gama=wo/2;
alfa=18.3;
delta=86;
beta=delta^2;

% Discretizao do sistema + zoh
GGzoh_z=c2d(G_s,T,'zoh');
% Discretizao do sistema + zoh + filtro anti-aliasing
GGfGzoh_z=c2d(G_s*Gf_s,T,'zoh');
% Discretizao do controlador recorrendo  transformao bilinear
K_z=c2d(K_s,T,'tustin')
% Sinal Sinusoidal com frequncia = 86 rad/s
N_s=alfa/(s^2+beta);
% Nota: Quando amostrado a 42 rad/s reverte-se num sinal com w=2 rad/s (depois de reconstrudo!) 
%
% Tranformada em estrela de Gf(s)*N(s), i.e. GfN(s)*=GfN_z
% Starred transform of Gf(s)*N(s), i.e. GfN(s)*=GfN_z
R1_z=((alfa*gama)/(gama^2+beta))*(z/(z-exp(-T*gama)));
R2_z=((alfa*gama*z)/(delta*(gama^2+delta^2)))*(gama*sin(T*delta)+delta*cos(T*delta)-delta*z)/(z^2-2*z*cos(T*delta)+1);
GfN_z=R1_z+R2_z
N_z=(alfa*z/delta)*(sin(T*delta)/(z^2-2*z*cos(T*delta)+1)); % Obtido por aplicao do teorema dos resduos a N(s)!
Yf_z=-GGzoh_z*K_z*GfN_z/(1+GGfGzoh_z*K_z);                  % Resposta com Filtro Anti-Aliasing...
Y_z=-(GGzoh_z*K_z*N_z)/(1+GGzoh_z*K_z);                     % Resposta sem Filtro Anti-Aliasing
Y_s=-G_s*K_s*N_s/(1+G_s*K_s);                               % Resposta do sistema puramente analgico  mesma perturbao
t=0:T:40;
[y_z]=impulse(Y_z,t);[y_zf]=impulse(Yf_z,t);[y_s]=impulse(Y_s,t);
subplot(2,1,1);plot(t,y_z,t,y_zf,t,y_s)
legend('Sem Pr-Filtro','Com Pr-Filtro','Analgico');xlabel('Tempo/s');ylabel('Amplitude')
subplot(2,3,4);plot(t,y_z);      % Sem pr-filtro
xlabel('Tempo/s');ylabel('Amplitude')
subplot(2,3,5);plot(t,y_zf);     % Com pr-filtro
xlabel('Tempo/s')
subplot(2,3,6);plot(t,y_s);      % Analgico
xlabel('Tempo/s')
